local modName, modTable = ...

local MAJOR, MINOR = "LibTradeSkillScan", tonumber("r221") or 1;
local Lib, oldminor = LibStub:NewLibrary(MAJOR, MINOR);
if not Lib then
	return -- No Upgrade needed.
end

local alwaysScan = true				-- to help debug
local DELAY_BEFORE_SCAN_STARTS = 5		-- wait 5 seconds for all mods to register

local maxLevel = 750
local maxSkillDepth = 10
local recipeColors = { {}, {}, {} }
local tradeIDList = {
	2018,			-- blacksmithing
	2108,			-- leatherworking
	25229,			-- jewelcrafting
	7411,			-- enchanting
	2259,			-- alchemy

	4036,			-- engineering
	45357,			-- inscription

	3908,			-- tailoring
	3273,			-- first aid
	2550,			-- cooking
--	2575,			-- smelting
}
local tradeIndex = 1

local progressBar
local showRegisteredFrames
local updateRegisteredFrames

local CloseTradeSkill
if ArmoryTradeSkillFrame then
	maxSkillDepth = 0
	CloseTradeSkill = function()
		ArmoryTradeSkillFrame.closing = nil
		_G.CloseTradeSkill()
	end
else
	CloseTradeSkill = _G.CloseTradeSkill
end


local OldGetTradeSkillReagentItemLink = GetTradeSkillReagentItemLink
local LTSSParsingTooltip = CreateFrame("GameTooltip", "LTSSParsingTooltip", ANCHOR_NONE, "GameTooltipTemplate")
LTSSParsingTooltip:SetOwner(WorldFrame, "ANCHOR_NONE")

local function GetTradeSkillReagentItemLink(i, j)
	if OldGetTradeSkillReagentItemLink(i, j) then
		-- GetTradeSkillReagentItemLink is working again, install the original
		print("LibTradeSkillScan: GetTradeSkillReagentItemLink() is working again")
		GetTradeSkillReagentItemLink = OldGetTradeSkillReagentItemLink
		return OldGetTradeSkillReagentItemLink(i, j)
	else
		local scanTooltip = _G["LTSSParsingTooltip"]
		scanTooltip:ClearLines()
		scanTooltip:SetTradeSkillItem(i, j)
		local _, reagentLink = scanTooltip:GetItem()

		return reagentLink
	end
end


local function InitProgressBar()
	if not progressBar then
		progressBar = CreateFrame("Frame", nil, UIParent)
		progressBar:SetFrameStrata("DIALOG")

		progressBar:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16,
			edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 }});
		progressBar:SetBackdropColor(0,0,0,1);

		progressBar:SetWidth(310)
		progressBar:SetHeight(30)

		progressBar:SetPoint("CENTER",0,-150)

		progressBar.fg = progressBar:CreateTexture()
		progressBar.fg:SetTexture(.8,.7,.2,.5)
		progressBar.fg:SetPoint("LEFT",progressBar,"LEFT",5,0)
		progressBar.fg:SetHeight(20)
		progressBar.fg:SetWidth(300)

		progressBar.textLeft = progressBar:CreateFontString(nil,"ARTWORK","GameFontNormalSmall")
		progressBar.textLeft:SetPoint("LEFT",10,0)

		progressBar.textRight = progressBar:CreateFontString(nil,"ARTWORK","GameFontNormalSmall")
		progressBar.textRight:SetPoint("RIGHT",-10,0)

		progressBar:EnableMouse()
	end
end


-- LEVEL SCANNING
do
	local scanner = {}

	LibStub("AceEvent-3.0"):Embed(scanner)
	LibStub("AceTimer-3.0"):Embed(scanner)


	local skillLevels = {
		unknown = 0,
		optimal = 1,
		medium = 2,
		trivial = 3,
	}

	local frame = CreateFrame("Frame")

	local level = 0
	local tradeLink
	local currentTrade
	local OpenTradeLink

	local recipeGray = {}

	local scanTimeout = false

	local scanCompleted = false

	local timeOutRetry

	local stackCount = 10

	local Init = function(data)
		print("this should actually do something other than print this")
	end


	local function InitTradeLinkTimer()
		progressBar.timeout = nil
		progressBar.timeoutCount = nil
		progressBar.timer = nil
		progressBar.dotCount = nil
	end


	local function InitTradeLink(self, elapsed)
		if not self.timeout then
			self.timeout = 10
		elseif self.timeout < 0 then
			local timeoutCount = self.timeoutCount or 1
			timeoutCount = timeoutCount + 1

			if timeoutCount > 10 then
				timeoutCount = 0
				scanTimeout = true
			else
				self.textRight:SetText("Try "..timeoutCount.." of 10")
				scanTimeout = false
			end

			scanner:ScheduleTimer(OpenTradeLink, 1)

			self.timeoutCount = timeoutCount
			self.timeout = 10
		else
			self.timeout = self.timeout - elapsed
		end

		if not self.timer then
			self.timer = 0.3
		elseif self.timer < 0 then
			local dotCount = self.dotCount or 0
			dotCount = dotCount + 1
			if ( dotCount > 3 ) then
				dotCount = 0
			end

			self.textLeft:SetText(currentTrade.." Retrieving"..(string.rep(".", dotCount)))
			self.dotCount = dotCount
			self.timer = 0.3
		else
			self.timer = self.timer - elapsed
		end
	end


	OpenTradeLink = function()
		progressBar:SetScript("OnUpdate", InitTradeLink)

		local tradeString = string.match(tradeLink, "(trade:%d+:%d+:%d+:[0-9a-fA-F]+:[A-Za-z0-9+/]+)")
		if tradeString and tradeLink then
			SetItemRef(tradeString,tradeLink,"LeftButton")
		end
	end


	local testGUID = ""


	function SetGUID()
		local guid = UnitGUID("target")
		testGUID = string.gsub(guid,"0x0+", "")..":"
	end

	local function OpenNextTradeSkill()
		level = level + 1

		if level > maxLevel then
			level = 1
			stackCount = 10
			tradeIndex = tradeIndex + 1
			currentTrade = GetSpellLink(tradeIDList[tradeIndex])
			if currentTrade then
				progressBar.textLeft:SetText(currentTrade.." Retrieving")
			end
			wipe(recipeGray)
		end

		local percentComplete = math.floor(level / maxLevel * 100)
		progressBar.fg:SetWidth(3 * percentComplete)
		progressBar.textRight:SetText(level.."/"..maxLevel.." ( "..percentComplete.."% )")

		if tradeIndex <= #tradeIDList then
			tradeLink = Lib:GetFullLink(tradeIDList[tradeIndex],testGUID,level)

			stackCount = stackCount - 1

			InitTradeLinkTimer()
			scanner:CancelAllTimers()
			scanner:ScheduleTimer(OpenTradeLink, 0.05)
		else
			scanCompleted = true

			for i, f in pairs(showRegisteredFrames) do
				f:RegisterEvent("TRADE_SKILL_SHOW")
			end

			for i, f in pairs(updateRegisteredFrames) do
				f:RegisterEvent("TRADE_SKILL_UPDATE")
			end

			frame:UnregisterEvent("TRADE_SKILL_SHOW")

			frame:Hide()
			progressBar:Hide()

			CloseTradeSkill()

			Init(recipeColors)
		end
	end


	local function TradeSkillShow()
		if scanTimeout or IsTradeSkillReady() then
			progressBar:SetScript("OnUpdate", nil)
			progressBar.textLeft:SetText(currentTrade.." Scanning...")
		else
			progressBar.fg:SetWidth(0)
			InitTradeLinkTimer()
			progressBar:SetScript("OnUpdate", InitTradeLink)
			return
		end

		SetTradeSkillCategoryFilter(-1)
		SetTradeSkillItemNameFilter(nil)
		SetTradeSkillItemLevelFilter(0, 0)
		SetTradeSkillInvSlotFilter(-1, 1, 1)
		TradeSkillOnlyShowSkillUps(false)
		TradeSkillOnlyShowMakeable(false)

		local name, skillType = GetTradeSkillInfo(1)
		local numTradeSkills = GetNumTradeSkills()

		if skillType == "header" or scanTimeout then
			InitTradeLinkTimer()
			local professionName, professionLevel = GetTradeSkillLine()
			for i = 1, numTradeSkills do
				if not recipeGray[i] then
					local skillName, skillColor = GetTradeSkillInfo(i)
					local recipeLink = GetTradeSkillRecipeLink(i)
					local skillLevel = skillLevels[skillColor]

					if skillColor == "trivial" then
						recipeGray[i] = true
					end

					if recipeLink then
						if skillLevel then
							local recipeID = tonumber(string.match(recipeLink, "enchant:(%d+)"))

							if skillLevel > 1 and not recipeColors[skillLevel][recipeID] then
								recipeColors[skillLevel][recipeID] = professionLevel
							end
						end
					else
						recipeGray[i] = true
					end
				end
			end

			OpenNextTradeSkill()
		else
			progressBar:SetScript("OnUpdate", InitTradeLink)

			for i = 1, numTradeSkills do
				for j = 1, GetTradeSkillNumReagents(i) do
					local reagentLink = GetTradeSkillReagentItemLink(i, j)

					if reagentLink then
						GameTooltip:SetHyperlink(reagentLink)
						GameTooltip:Show()
					end
				end
			end
		end
	end


	function Lib:ScanLevels(guid, initFunction)
		if initFunction then
			Init = initFunction
		end

		tradeIndex = 1
		level = 0
		currentTrade = GetSpellLink(tradeIDList[tradeIndex])

		testGUID = guid

		showRegisteredFrames = { GetFramesRegisteredForEvent("TRADE_SKILL_SHOW") }
		for i,f in pairs(showRegisteredFrames) do
			f:UnregisterEvent("TRADE_SKILL_SHOW")
		end

		updateRegisteredFrames = { GetFramesRegisteredForEvent("TRADE_SKILL_UPDATE") }
		for i,f in pairs(updateRegisteredFrames) do
			f:UnregisterEvent("TRADE_SKILL_UPDATE")
		end

		InitProgressBar()

		progressBar:SetScript("OnEnter", function(frame)
			GameTooltip:ClearLines()
			GameTooltip:SetOwner(frame, "ANCHOR_NONE")
			GameTooltip:SetPoint("BOTTOM",frame,"TOP",0,5)
			GameTooltip:AddLine("Scanning trade skill levels")
			GameTooltip:Show()
		end)

		progressBar:SetScript("OnLeave", function(frame)
			GameTooltip:Hide()
		end)

		progressBar.textLeft:SetText(currentTrade)
		progressBar.textRight:SetText("")
		progressBar:Show()

		frame:RegisterEvent("TRADE_SKILL_SHOW")

		frame:SetScript("OnEvent", function(f, event)
			if event == "TRADE_SKILL_SHOW" then
				TradeSkillShow()
			end
		end)

		scanner:ScheduleTimer(OpenNextTradeSkill, 1)
	end
end


do
	local AddonRegistry = {}
	local scanInitiated
	local scanComplete

	local clientVersion, clientBuild = GetBuildInfo()
	clientBuild = tonumber(clientBuild)
	local minBuild = 0 -- clientBuild


	local fullTradeLink = {}


	local startTime
	local startMem

	local encodedByte = {
							'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
							'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
							'0','1','2','3','4','5','6','7','8','9','+','/'
						}

	local decodedByte = {}

	for i=1,#encodedByte do
		local b = string.byte(encodedByte[i])

		decodedByte[b] = i - 1
	end


	local tradeIndexByID = {}

	for k,tradeID in ipairs(tradeIDList) do
		tradeIndexByID[tradeID] = k
	end


	local tradeIDByName = {}

	for index, id in pairs(tradeIDList) do
		local tradeName = string.lower(GetSpellInfo(id))
		tradeIDByName[tradeName] = id
	end

	tradeIDByName[string.lower(GetSpellInfo(2575))] = 2656	-- special case for mining/smelting





	local playerGUID

	local spellList = {}


	local spellBit = 0
	local countDown = 5
	local bitMapSizes = {}
	local timeToClose = 0
	local frameOpen = false

	local function DispatchInitialization()
		for addonName, addon in pairs(AddonRegistry) do
			if not addon.initialized then
				addon.init(spellList)
				addon.initialized = true
			end
		end
	end


	local function GenerateFullLinks()
		for tradeIndex, tradeID in pairs(tradeIDList) do
			local spellLink, tradeLink = GetSpellLink(tradeID)

			if not tradeLink then
				return
			end

			local tradeHead, oldbitMap = string.match(tradeLink, "(|cffffd000|Htrade:%d+:%d+:%d+:[0-9A-Fa-f]+:)([0-9a-zA-Z+/]+)")

			local bitMap = Lib:BitmapFull(spellList[tradeID])


			local fullLink = string.format("%s%s|h[%s]|h|r", tradeHead,bitMap,GetSpellInfo(tradeID))

--	print(fullLink, string.gsub(fullLink, "|", "||"))

			fullTradeLink[tradeID] = fullLink
		end
	end


	local function ScanComplete(frame)

		frame:SetScript("OnUpdate", nil)
		frame:UnregisterEvent("TRADE_SKILL_UPDATE")
		frame:UnregisterEvent("TRADE_SKILL_CLOSE")
		frame:UnregisterEvent("TRADE_SKILL_SHOW")

		frame:Hide()

		for k, f in pairs(showRegisteredFrames) do
			f:RegisterEvent("TRADE_SKILL_SHOW")
		end

		progressBar:Hide()

		GenerateFullLinks()

		scanComplete = true

		DispatchInitialization()
	end



	local function OnTradeSkillShow()
		frameOpen = true
	end


	local function OnTradeSkillClose(frame)
		frameOpen = false

		spellBit = spellBit + 1

		if spellBit <= (bitMapSizes[tradeIndex] or 0)*6 then
			local percentComplete = spellBit/(bitMapSizes[tradeIndex]*6)

			progressBar.fg:SetWidth(300*percentComplete)
			progressBar.textRight:SetText(spellBit)


			local bytes = floor((spellBit-1)/6)
			local bits = (spellBit-1) - bytes*6

			local bmap = string.rep("A", bytes) .. encodedByte[bit.lshift(1, bits)+1] .. string.rep("A", bitMapSizes[tradeIndex]-bytes-1)

			local tradeString = string.format("trade:%d:%d:%d:%s:%s", tradeIDList[tradeIndex], maxLevel, maxLevel, playerGUID, bmap)

			local link = "|cffffd000|H"..tradeString.."|h["..GetSpellInfo(tradeIDList[tradeIndex]).."]|h|r"

			timeToClose = 30 / 3

			ItemRefTooltip:SetHyperlink(tradeString)
		else
			tradeIndex = tradeIndex + 1
			spellBit = 0

			if tradeIndex <= #tradeIDList then
				OnTradeSkillClose(frame)
			else
				ScanComplete(frame)
			end
		end
	end


	local tradeSkillDepth = maxSkillDepth

	local function OnTradeSkillUpdate(frame)
		if spellBit > 0 and bitMapSizes[tradeIndex] then
			if not spellList[tradeIDList[tradeIndex]] then
				spellList[tradeIDList[tradeIndex]] = {}
			end

			local numSkills = GetNumTradeSkills()

			spellList[tradeIDList[tradeIndex]][spellBit] = tradeIDList[tradeIndex] -- placeHolder

			local recipeLink = GetTradeSkillRecipeLink(numSkills)

			if recipeLink then
				local recipeID = tonumber(recipeLink:match("enchant:(%d+)"))

				progressBar.textLeft:SetText(recipeLink)
				spellList[tradeIDList[tradeIndex]][spellBit] = recipeID
			end

			if tradeSkillDepth < maxSkillDepth then
				tradeSkillDepth = tradeSkillDepth + 1
				CloseTradeSkill()
			else
				tradeSkillDepth = 0
				timeToClose = 0.001
			end
		else
			timeToClose = 0
		end
	end


	local function OnUpdate(frame, elapsed)
		timeToClose = timeToClose - elapsed

		if timeToClose < 0 then
			timeToClose = 1000
			CloseTradeSkill()
		end
	end


	function Lib:Scan()
		startTime = time()

		local guid = UnitGUID("player")
		playerGUID =  string.gsub(guid,"0x0+", "")
		tradeIndex = 1

		showRegisteredFrames = { GetFramesRegisteredForEvent("TRADE_SKILL_SHOW") }
		for k, f in pairs(showRegisteredFrames) do
			f:UnregisterEvent("TRADE_SKILL_SHOW")
		end

		for k,id in ipairs(tradeIDList) do
			ItemRefTooltip:SetHyperlink(fullTradeLink[id])
		end

		InitProgressBar()

		progressBar:SetScript("OnEnter", function(frame)
			GameTooltip:ClearLines()
			GameTooltip:SetOwner(frame, "ANCHOR_NONE")
			GameTooltip:SetPoint("BOTTOM",frame,"TOP",0,5)

			GameTooltip:AddLine("Scanning recipe data for the following addons:")

			for addon in pairs(AddonRegistry) do
				GameTooltip:AddLine(addon)
			end

			GameTooltip:Show()
		end)

		progressBar:SetScript("OnLeave", function(frame)
			GameTooltip:Hide()
		end)

		local scanFrame = CreateFrame("Frame")


		scanFrame:RegisterEvent("TRADE_SKILL_SHOW")
		scanFrame:RegisterEvent("TRADE_SKILL_UPDATE")
		scanFrame:RegisterEvent("TRADE_SKILL_CLOSE")

		scanFrame:SetScript("OnEvent", function(frame,event)
--DEFAULT_CHAT_FRAME:AddMessage(tostring(event))
			if event == "TRADE_SKILL_SHOW" then
				OnTradeSkillShow(frame)
			end

			if event == "TRADE_SKILL_CLOSE" then
				OnTradeSkillClose(frame)
			end

			if event == "TRADE_SKILL_UPDATE" then
				OnTradeSkillUpdate(frame)
			end
		end)

		scanFrame:SetScript("OnUpdate", OnUpdate)


		for tradeIndex, tradeID in ipairs(tradeIDList) do

			local _,tradeLink = GetSpellLink(tradeID)

			local bitMap = string.match(tradeLink,"|c%x+|Htrade:%d+:%d+:%d+:[0-9a-fA-F]+:([A-Za-z0-9+/]+)|h%[[^]]+%]|h|r")

			bitMapSizes[tradeIndex] = string.len(bitMap)
		end


		OnTradeSkillClose()
	end


	function Lib:BitmapFull(data)
		local v = 0
		local b = 1
		local bitmap = ""

		if data then
			for i=1,#data do
				if data[i] then
					v = v + b
				end

				b = b * 2

				if b == 64 then
					bitmap = bitmap .. encodedByte[v+1]
					v = 0
					b = 1
				end
			end

			if b>1 then
				bitmap = bitmap .. encodedByte[v+1]
			end
		end

		return bitmap
	end


	function Lib:BitmapEncode(data, mask)
		local v = 0
		local b = 1
		local bitmap = ""

		if data then
			for i=1,#data do
				if mask[data[i]] == true then
					v = v + b
				end

				b = b * 2

				if b == 64 then
					bitmap = bitmap .. encodedByte[v+1]
					v = 0
					b = 1
				end
			end

			if b>1 then
				bitmap = bitmap .. encodedByte[v+1]
			end
		end

		return bitmap
	end


	function Lib:BitmapDecode(data, bitmap, maskTable)
		local mask = maskTable or {}
		local index = 1

		for i=1, string.len(bitmap) do
			local b = decodedByte[string.byte(bitmap, i)]
			local v = 1

			for j=1,6 do
				if index <= #data and data[index] then
					if bit.band(v, b) == v then
						mask[data[index]] = true
					else
						mask[data[index]] = false
					end
				end
				v = v * 2

				index = index + 1
			end
		end

		return mask
	end


	function Lib:BitmapBitLogic(A,B,logic)
		local length = math.min(string.len(A), string.len(B))
		local R = ""

		for i=1, length do
			local a = decodedByte[string.byte(A, i)]
			local b = decodedByte[string.byte(B, i)]

			local r = logic(a,b)

			R = R..encodedByte[r+1]
		end

		return R
	end


	function Lib:DumpSpells(data, bitmap)
		local index = 1

		for i=1, string.len(bitmap) do
			local b = decodedByte[string.byte(bitmap, i)]
			local v = 1

			for j=1,6 do
				if index <= #data then
					if bit.band(v, b) == v then
						DEFAULT_CHAT_FRAME:AddMessage("bit "..index.." = spell:"..data[index].." "..GetSpellLink(data[index]))
					end
				end
				v = v * 2

				index = index + 1
			end
		end
	end



	function Lib:BitmapCompress(bitmap)
		if not bitmap then return end

		local len = string.len(bitmap)
		local compressed = {}
		local n = 1

		for i=1,len,5 do
			local map = 0

			map = decodedByte[string.byte(bitmap, i) or 65]

			v = decodedByte[string.byte(bitmap,i+1) or 65]
			map = bit.lshift(map, 6) + v


			v = decodedByte[string.byte(bitmap,i+2) or 65]
			map = bit.lshift(map, 6) + v


			v = decodedByte[string.byte(bitmap,i+3) or 65]
			map = bit.lshift(map, 6) + v


			v = decodedByte[string.byte(bitmap,i+4) or 65]
			map = bit.lshift(map, 6) + v

			compressed[n] = map

			n = n + 1
		end

		return compressed
	end



	-- must pass the guid
	function Lib:GenerateLink(guid)
		if guid and GetTradeSkillLine() then
			local tradeName, rank, maxRank = GetTradeSkillLine()

			local tradeID = tradeIDByName[string.lower(tradeName)]

			local bitMapSize = bitMapSizes[tradeIndexByID[tradeID]] or 0

			local mask = {}

			for i=1,bitMapSize*6 do
				local spellName, spellType = GetTradeSkillInfo(i)

				if spellName and spellType ~= "header" then
					local recipeLink = GetTradeSkillRecipeLink(i)
					local recipeID = tonumber(recipeLink:match("enchant:(%d+)"))

					mask[recipeID] = true
				end
			end


			local bitMap = Lib:BitmapEncode(spellList[tradeID], mask)

			local tradeLink = string.format("|cffffd000|Htrade:%d:%d:%d:%s:%s|h[%s]|h|r", tradeID, rank, maxRank, guid, bitMap,tradeName)

			return tradeLink
		end
	end


	function Lib:GetFullLink(tradeID,guid,level)
--		if not fullTradeLink[tradeID] then
--			GenerateFullLinks()
--		end


		if not playerGUID then
			local guid = UnitGUID("player")
			playerGUID =  string.gsub(guid,"0x0+", "")
		end

		local bitMap = Lib:BitmapFull(spellList[tradeID])

		local tradeLink = string.format("|cffffd000|Htrade:%d:%d:%d:%s:%s|h[%s]|h|r", tradeID, level or maxLevel, maxLevel, guid or playerGUID, bitMap, (GetSpellInfo(tradeID)))

		return tradeLink
	end


-- the following only operate on COMPRESSED bitmaps
	function Lib:BitsShared(b1, b2)
		local sharedBits = 0
		local len = math.min(#b1,#b2)

		for i=1,len do
			result = bit.band(b1[i],b2[i] or 0)

			if result~=0 then
				for b=0,29 do
					if bit.band(result, 2^b)~=0 then
						sharedBits = sharedBits + 1
					end
				end
			end
		end

		return sharedBits
	end


	function Lib:CountBits(bmap)
		local bits = 0
		local len = #bmap

		for i=1,len do
			if result~=0 then
				for b=0,29 do
					if bit.band(bmap[i], 2^b)~=0 then
						bits = bits + 1
					end
				end
			end
		end
		return bits
	end



	local function waitForSpellLinks()
		for tradeIndex, tradeID in pairs(tradeIDList) do
			local spellLink, tradeLink = GetSpellLink(tradeID)

			if not tradeLink then
				return
			end

			local tradeHead, bitMap = string.match(tradeLink, "(|cffffd000|Htrade:%d+:%d+:%d+:[0-9A-Fa-f]+:)([0-9a-zA-Z+/]+)")
			local len = string.len(bitMap)

			local tradeString = tradeHead..string.rep("/",len)								-- quickly generate a full link which includes some potentially bad data, but will help seed the scanner

			local fullLink = tradeString.."|h["..GetSpellInfo(tradeID).."]|h|r"

--print(fullLink, string.gsub(fullLink, "|", "||"))

			fullTradeLink[tradeID] = fullLink
		end

		return true
	end




	function Lib:Register(addonName, initFunction, build, data)
		build = build or 0

		minBuild = max(build, minBuild)

		if build == clientBuild and spellList and not scanInitiated then
			spellList = data								-- copy this data!
		end

		AddonRegistry[addonName] = { init = initFunction, build = build, data = data }

		if alwaysScan or minBuild ~= clientBuild then
			if not scanInitiated then
				scanInitiated = true

				local delayFrame = CreateFrame("Frame")
				delayFrame.timer = DELAY_BEFORE_SCAN_STARTS
				delayFrame:SetScript("OnUpdate", function(frame, elapsed)
					frame.timer = frame.timer - elapsed

					if frame.timer < 0 then
						if waitForSpellLinks() then
							frame:Hide()
							frame:SetScript("OnUpdate",nil)

							if minBuild ~= clientBuild then
								Lib:Scan()
							else
								DispatchInitialization()
							end
							frame.timer = 100000
						else
							frame.timer = 1
						end
					end
				end)
			else
				if scanComplete then
					initFunction(spellList)
					AddonRegistry[addonName].initialized = true
				end
			end
		else
			initFunction(spellList)
			AddonRegistry[addonName].initialized = true
		end
	end

--[[
	function Lib:Cleanup()
		TradeSkillData = {}
		recipeData = nil
	end
--]]

end

